@@ -1,7 +1,8 @@
 import unittest
 from igraph.test import basic, layouts, games, foreign, structural, flow, \
     spectral, attributes, cliques, decomposition, operators, generators, \
-    isomorphism, colortests, vertexseq, edgeseq, iterators, bipartite
+    isomorphism, colortests, vertexseq, edgeseq, iterators, bipartite, \
+    conversion, rng, separators, indexing, atlas, matching
 
 def suite():
     return unittest.TestSuite( \
@@ -18,21 +19,31 @@ def suite():
          edgeseq.suite(),
          cliques.suite(),
          decomposition.suite(),
+         conversion.suite(),
          operators.suite(),
          isomorphism.suite(),
          iterators.suite(),
          bipartite.suite(),
-         colortests.suite()] \
+         colortests.suite(),
+         rng.suite(),
+         separators.suite(),
+         indexing.suite(),
+         atlas.suite(),
+         matching.suite()] \
     )
     
-def test():
+def run_tests(verbosity=1):
     try:
         # Support for testoob to have nice colored output
         import testoob
-        testoob.main(suite())
+        runner = testoob.main
     except ImportError:
-        runner = unittest.TextTestRunner(verbosity=2)
-        runner.run(suite())
+        runner = unittest.TextTestRunner(verbosity=verbosity).run
+    runner(suite())
 
-if __name__ == "__main__": test()
+# Make nosetest skip run_tests
+run_tests.__test__ = False
+
+if __name__ == "__main__":
+    run_tests(255)
 
