@@ -1,6 +1,7 @@
 import unittest
-from igraph import *
 
+from igraph import *
+from igraph.compat import isnan
 
 class SimplePropertiesTests(unittest.TestCase):
     gfull  = Graph.Full(10)
@@ -41,11 +42,49 @@ class SimplePropertiesTests(unittest.TestCase):
         d = self.tree.farthest_points(weights=weights)
         self.failUnless(d == (13, 6, 15) or d == (6, 13, 15))
 
+    def testEccentricity(self):
+        self.assertEquals(self.gfull.eccentricity(),
+                [1] * self.gfull.vcount())
+        self.assertEquals(self.gempty.eccentricity(),
+                [0] * self.gempty.vcount())
+        self.assertEquals(self.g.eccentricity(), [1, 1, 2, 2])
+        self.assertEquals(self.gdir.eccentricity(),
+                [1, 2, 3, 2])
+        self.assertEquals(self.tree.eccentricity(),
+                [3, 3, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 5])
+        self.assertEquals(Graph().eccentricity(), [])
+
+    def testRadius(self):
+        self.assertEquals(self.gfull.radius(), 1)
+        self.assertEquals(self.gempty.radius(), 0)
+        self.assertEquals(self.g.radius(), 1)
+        self.assertEquals(self.gdir.radius(), 1)
+        self.assertEquals(self.tree.radius(), 3)
+        self.assertTrue(isnan(Graph().radius()))
+
     def testTransitivity(self):
         self.failUnless(self.gfull.transitivity_undirected() == 1.0)
         self.failUnless(self.tree.transitivity_undirected() == 0.0)
         self.failUnless(self.g.transitivity_undirected() == 0.75)
 
+    def testLocalTransitivity(self):
+        self.failUnless(self.gfull.transitivity_local_undirected() ==
+                [1.0] * self.gfull.vcount())
+        self.failUnless(self.tree.transitivity_local_undirected(mode="zero") ==
+                [0.0] * self.tree.vcount())
+
+        l = self.g.transitivity_local_undirected(mode="zero")
+        self.assertAlmostEqual(l[0], 2/3., places=4)
+        self.assertAlmostEqual(l[1], 2/3., places=4)
+        self.assertEquals(l[2], 1)
+        self.assertEquals(l[3], 1)
+
+        g = Graph.Full(4) + 1 + [(0, 4)]
+        g.es["weight"] = [1, 1, 1, 1, 1, 1, 5]
+        self.assertAlmostEqual(
+                g.transitivity_local_undirected(0, weights="weight"),
+                0.25, places=4)
+
     def testAvgLocalTransitivity(self):
         self.failUnless(self.gfull.transitivity_avglocal_undirected() == 1.0)
         self.failUnless(self.tree.transitivity_avglocal_undirected() == 0.0)
@@ -71,17 +110,36 @@ class DegreeTests(unittest.TestCase):
     gdir = Graph(4, [(0, 1), (0, 2), (1, 2), (2, 1), (0, 3), (1, 3), (3, 0)], directed=True)
     tree = Graph.Tree(10, 3)
 
+    def testKnn(self):
+        knn, knnk = self.gfull.knn()
+        self.failUnless(knn == [9.] * 10)
+        self.assertAlmostEquals(knnk[8], 9.0, places=6)
+
+        # knn works for simple graphs only -- self.g is not simple
+        self.assertRaises(InternalError, self.g.knn)
+
+        # Okay, simplify it and then go on
+        g = self.g.copy()
+        g.simplify()
+
+        knn, knnk = g.knn()
+        diff = max(abs(a-b) for a, b in zip(knn, [7/3., 7/3., 3, 3]))
+        self.assertAlmostEquals(diff, 0., places=6)
+        self.assertEquals(len(knnk), 3)
+        self.assertAlmostEquals(knnk[1], 3, places=6)
+        self.assertAlmostEquals(knnk[2], 7/3., places=6)
+
     def testDegree(self):
         self.failUnless(self.gfull.degree() == [9] * 10)
         self.failUnless(self.gempty.degree() == [0] * 10)
         self.failUnless(self.g.degree(loops=False) == [3, 3, 2, 2])
         self.failUnless(self.g.degree() == [5, 3, 2, 2])
-        self.failUnless(self.gdir.degree(type=IN) == [1, 2, 2, 2])
-        self.failUnless(self.gdir.degree(type=OUT) == [3, 2, 1, 1])
-        self.failUnless(self.gdir.degree(type=ALL) == [4, 4, 3, 3])
+        self.failUnless(self.gdir.degree(mode=IN) == [1, 2, 2, 2])
+        self.failUnless(self.gdir.degree(mode=OUT) == [3, 2, 1, 1])
+        self.failUnless(self.gdir.degree(mode=ALL) == [4, 4, 3, 3])
         vs = self.gdir.vs.select(0, 2)
-        self.failUnless(self.gdir.degree(vs, type=ALL) == [4, 3])
-        self.failUnless(self.gdir.degree(self.gdir.vs[1], type=ALL) == 4)
+        self.failUnless(self.gdir.degree(vs, mode=ALL) == [4, 3])
+        self.failUnless(self.gdir.degree(self.gdir.vs[1], mode=ALL) == 4)
 
     def testMaxDegree(self):
         self.failUnless(self.gfull.maxdegree() == 9)
@@ -89,9 +147,9 @@ class DegreeTests(unittest.TestCase):
         self.failUnless(self.g.maxdegree() == 3)
         self.failUnless(self.g.maxdegree(loops=True) == 5)
         self.failUnless(self.g.maxdegree([1, 2], loops=True) == 3)
-        self.failUnless(self.gdir.maxdegree(type=IN) == 2)
-        self.failUnless(self.gdir.maxdegree(type=OUT) == 3)
-        self.failUnless(self.gdir.maxdegree(type=ALL) == 4)
+        self.failUnless(self.gdir.maxdegree(mode=IN) == 2)
+        self.failUnless(self.gdir.maxdegree(mode=OUT) == 3)
+        self.failUnless(self.gdir.maxdegree(mode=ALL) == 4)
         
     def testStrength(self):
         # Turn off warnings about calling strength without weights
@@ -110,17 +168,17 @@ class DegreeTests(unittest.TestCase):
                 [7, 9, 5, 9])
         self.failUnless(self.g.strength(weights=ws) == [19, 9, 5, 9])
         ws = [1, 2, 3, 4, 5, 6, 7]
-        self.failUnless(self.gdir.strength(type=IN, weights=ws) == \
+        self.failUnless(self.gdir.strength(mode=IN, weights=ws) == \
                 [7, 5, 5, 11])
-        self.failUnless(self.gdir.strength(type=OUT, weights=ws) == \
+        self.failUnless(self.gdir.strength(mode=OUT, weights=ws) == \
                 [8, 9, 4, 7])
-        self.failUnless(self.gdir.strength(type=ALL, weights=ws) == \
+        self.failUnless(self.gdir.strength(mode=ALL, weights=ws) == \
                 [15, 14, 9, 18])
         vs = self.gdir.vs.select(0, 2)
-        self.failUnless(self.gdir.strength(vs, type=ALL, weights=ws) == \
+        self.failUnless(self.gdir.strength(vs, mode=ALL, weights=ws) == \
                 [15, 9])
         self.failUnless(self.gdir.strength(self.gdir.vs[1], \
-                type=ALL, weights=ws) == 14)
+                mode=ALL, weights=ws) == 14)
 
         
 
@@ -187,6 +245,10 @@ class CentralityTests(unittest.TestCase):
         self.failUnless(g.edge_betweenness() == [6., 4., 4., 4.])
         self.failUnless(g.edge_betweenness(cutoff=2) == [4., 3., 3., 2.])
         self.failUnless(g.edge_betweenness(cutoff=1) == [1., 1., 1., 1.])
+        g = Graph.Ring(5)
+        self.failUnless(g.edge_betweenness() == [3., 3., 3., 3., 3.])
+        self.failUnless(g.edge_betweenness(weights=[4, 1, 1, 1, 1]) == \
+                [0.5, 3.5, 5.5, 5.5, 3.5])
 
     def testClosenessCentrality(self):
         g = Graph.Star(5)
@@ -201,12 +263,37 @@ class CentralityTests(unittest.TestCase):
         for idx in xrange(g.vcount()):
             self.assertAlmostEqual(cl[idx], cl2[idx], places=3)
 
+        weights = [1] * 4
+
+        g = Graph.Star(5)
+        cl = g.closeness(weights=weights)
+        cl2 = [1., 0.57142, 0.57142, 0.57142, 0.57142]
+        for idx in xrange(g.vcount()):
+            self.assertAlmostEqual(cl[idx], cl2[idx], places=3)
+
+        g = Graph.Star(5)
+        cl = g.closeness(cutoff=1, weights=weights)
+        cl2 = [1., 0.25, 0.25, 0.25, 0.25]
+        for idx in xrange(g.vcount()):
+            self.assertAlmostEqual(cl[idx], cl2[idx], places=3)
+
     def testPageRank(self):
         g = Graph.Star(11)
         cent = g.pagerank()
         self.failUnless(cent.index(max(cent)) == 0)
         self.assertAlmostEquals(max(cent), 0.4668, places=3)
 
+    def testPersonalizedPageRank(self):
+        g = Graph.Star(11)
+        self.assertRaises(InternalError, g.personalized_pagerank, reset=[0]*11)
+        cent = g.personalized_pagerank(reset=[0,10]+[0]*9, damping=0.5)
+        self.failUnless(cent.index(max(cent)) == 1)
+        self.assertAlmostEquals(cent[0], 0.3333, places=3)
+        self.assertAlmostEquals(cent[1], 0.5166, places=3)
+        self.assertAlmostEquals(cent[2], 0.0166, places=3)
+        cent2 = g.personalized_pagerank(reset_vertices=g.vs[1], damping=0.5)
+        self.failUnless(max(abs(x-y) for x, y in zip(cent, cent2)) < 0.001)
+
     def testEigenvectorCentrality(self):
         g = Graph.Star(11)
         cent = g.evcent()
@@ -238,6 +325,23 @@ class CentralityTests(unittest.TestCase):
         self.assertEquals(g.coreness("A"), [3,3,3,3,1,1,1,1])
 
 
+class NeighborhoodTests(unittest.TestCase):
+    def testNeighborhood(self):
+        g = Graph.Ring(10, circular=False)
+        self.failUnless(map(sorted, g.neighborhood()) == \
+                [[0,1], [0,1,2], [1,2,3], [2,3,4], [3,4,5], [4,5,6], \
+                    [5,6,7], [6,7,8], [7,8,9], [8,9]])
+        self.failUnless(map(sorted, g.neighborhood(order=3)) == \
+                [[0,1,2,3], [0,1,2,3,4], [0,1,2,3,4,5], [0,1,2,3,4,5,6], \
+                    [1,2,3,4,5,6,7], [2,3,4,5,6,7,8], [3,4,5,6,7,8,9], \
+                    [4,5,6,7,8,9], [5,6,7,8,9], [6,7,8,9]])
+
+    def testNeighborhoodSize(self):
+        g = Graph.Ring(10, circular=False)
+        self.failUnless(g.neighborhood_size() == [2,3,3,3,3,3,3,3,3,2])
+        self.failUnless(g.neighborhood_size(order=3) == [4,5,6,7,7,7,7,6,5,4])
+
+
 class MiscTests(unittest.TestCase):
     def testConstraint(self):
         g = Graph(4, [(0, 1), (0, 2), (1, 2), (0, 3), (1, 3)])
@@ -250,6 +354,18 @@ class MiscTests(unittest.TestCase):
         g.to_undirected()
         self.assertRaises(InternalError, g.topological_sorting)
 
+    def testIsDAG(self):
+        g = Graph(5, [(0, 1), (0, 2), (1, 2), (1, 3), (2, 3)], directed=True)
+        self.failUnless(g.is_dag())
+        g.to_undirected()
+        self.failIf(g.is_dag())
+        g = Graph.Barabasi(1000, 2, directed=True)
+        self.failUnless(g.is_dag())
+        g = Graph.GRG(100, 0.2)
+        self.failIf(g.is_dag())
+        g = Graph.Ring(10, directed=True, mutual=False)
+        self.failIf(g.is_dag())
+
     def testLineGraph(self):
         g = Graph(4, [(0, 1), (0, 2), (1, 2), (0, 3), (1, 3)])
         el = g.linegraph().get_edgelist()
@@ -284,6 +400,64 @@ class PathTests(unittest.TestCase):
         ]
         self.failUnless(g.shortest_paths(weights=ws) == expected)
         self.failUnless(g.shortest_paths(weights="weight") == expected)
+        self.failUnless(g.shortest_paths(weights="weight", target=[2,3]) ==
+                [row[2:4] for row in expected])
+
+    def testGetShortestPaths(self):
+        g = Graph(4, [(0,1), (0,2), (1,3), (3,2), (2,1)], directed=True)
+        sps = g.get_shortest_paths(0)
+        expected = [[0], [0, 1], [0, 2], [0, 1, 3]]
+        self.failUnless(sps == expected)
+        sps = g.get_shortest_paths(0, output="vpath")
+        expected = [[0], [0, 1], [0, 2], [0, 1, 3]]
+        self.failUnless(sps == expected)
+        sps = g.get_shortest_paths(0, output="epath")
+        expected = [[], [0], [1], [0, 2]]
+        self.failUnless(sps == expected)
+        self.assertRaises(ValueError, g.get_shortest_paths, 0, output="x")
+
+    def testGetAllShortestPaths(self):
+        g = Graph(4, [(0,1), (1, 2), (1, 3), (2, 4), (3, 4), (4, 5)], directed=True)
+
+        sps = sorted(g.get_all_shortest_paths(0, 5))
+        expected = [[0, 1, 2, 4, 5], [0, 1, 3, 4, 5]]
+        self.failUnless(sps == expected)
+
+        sps = sorted(g.get_all_shortest_paths(1, 4))
+        expected = [[1, 2, 4], [1, 3, 4]]
+        self.failUnless(sps == expected)
+
+        g = Graph.Lattice([5, 5], circular=False)
+        
+        sps = sorted(g.get_all_shortest_paths(0, 12))
+        expected = [[0, 1, 2, 7, 12], [0, 1, 6, 7, 12], [0, 1, 6, 11, 12], \
+                    [0, 5, 6, 7, 12], [0, 5, 6, 11, 12], [0, 5, 10, 11, 12]]
+        self.failUnless(sps == expected)
+
+        g = Graph.Lattice([100, 100], circular=False)
+        sps = sorted(g.get_all_shortest_paths(0, 202))
+        expected = [[0, 1, 2, 102, 202], [0, 1, 101, 102, 202], [0, 1, 101, 201, 202], \
+                    [0, 100, 101, 102, 202], [0, 100, 101, 201, 202], [0, 100, 200, 201, 202]]
+        self.failUnless(sps == expected)
+
+        g = Graph.Lattice([100, 100], circular=False)
+        sps = sorted(g.get_all_shortest_paths(0, [0, 202]))
+        self.failUnless(sps == [[0]] + expected)
+
+        g = Graph([(0,1), (1,2), (0,2)])
+        g.es["weight"] = [0.5, 0.5, 1]
+        sps = sorted(g.get_all_shortest_paths(0, weights="weight"))
+        self.failUnless(sps == [[0], [0,1], [0,1,2], [0,2]])
+
+        g = Graph.Lattice([4, 4], circular=False)
+        g.es["weight"] = 1
+        g.es[2,8]["weight"] = 100
+        sps = sorted(g.get_all_shortest_paths(0, [3, 12, 15], weights="weight"))
+        self.failUnless(len(sps) == 20)
+        self.failUnless(sum(1 for path in sps if path[-1] == 3) == 4)
+        self.failUnless(sum(1 for path in sps if path[-1] == 12) == 4)
+        self.failUnless(sum(1 for path in sps if path[-1] == 15) == 12)
+
 
     def testPathLengthHist(self):
         g = Graph.Tree(15, 2)
@@ -306,6 +480,7 @@ def suite():
     local_transitivity_suite = unittest.makeSuite(LocalTransitivityTests)
     biconnected_suite = unittest.makeSuite(BiconnectedComponentTests)
     centrality_suite = unittest.makeSuite(CentralityTests)
+    neighborhood_suite = unittest.makeSuite(NeighborhoodTests)
     path_suite = unittest.makeSuite(PathTests)
     misc_suite = unittest.makeSuite(MiscTests)
     return unittest.TestSuite([simple_suite,
@@ -313,6 +488,7 @@ def suite():
                                local_transitivity_suite,
                                biconnected_suite,
                                centrality_suite,
+                               neighborhood_suite,
                                path_suite,
                                misc_suite])
 
